'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _log = require('./log');

var log = _interopRequireWildcard(_log);

var _db = require('./db');

var _db2 = _interopRequireDefault(_db);

var _FBClient = require('./FBClient');

var _FBClient2 = _interopRequireDefault(_FBClient);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _refresher = require('./refresher');

var _refresher2 = _interopRequireDefault(_refresher);

var _events = require('events');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * The query parameters used to poll a group.
 * @constant {Object}
 */
var REFRESH_PARAMS = {

  fields: ['from', 'message', 'type', 'created_time', 'place', 'message_tags', 'link', 'caption', 'description', 'picture', 'comments.limit(1000){' + ['from', 'message', 'created_time', 'message_tags', 'comments.limit(1000){' + ['from', 'message', 'created_time', 'message_tags'].join() + '}'].join() + '}'].join(),

  limit: 10

};

function versionHash(post) {
  var s = post.id + '|' + post.from + '|' + (post.message || '');
  return _crypto2.default.createHash('sha1').update(s).digest('hex');
}

function messageCommand(message) {
  if (message == null) {
    return null;
  }
  message = message.trim();
  if (message.indexOf('/') === 0) {
    return message.split(/\s/g)[0].substring(1);
  } else {
    return null;
  }
}

function normalizeResponse(raw, group) {

  function normalizePost(raw) {

    var post = {
      id: raw.id,
      level: 0,
      group: group,
      parent: group,
      from: raw.from == null ? '' : raw.from.id,
      fromName: raw.from == null ? '' : raw.from.name,
      type: raw.type || null,
      message: raw.message || '',
      created: new Date(raw.created_time),
      place: raw.place || null,
      tags: raw.message_tags || [],
      link: raw.link == null ? null : {
        url: raw.link,
        caption: raw.caption,
        description: raw.description,
        picture: raw.picture
      },
      comments: [],
      isMod: group.mods.indexOf(raw.from.id) !== -1,
      command: messageCommand(raw.message)
    };

    post.version = versionHash(post);

    // Recurse over the post's comments
    if (raw.comments != null) {
      post.comments = raw.comments.data.map(function (raw) {
        var c = normalizePost(raw);
        c.type = 'comment';
        c.parent = post;
        c.level = post.level + 1;
        return c;
      });
    }

    return post;
  }

  return raw.data.map(normalizePost);
}

function walkThread(thread, fn) {

  function walk(post) {
    var s = [fn(post)];
    var r = (post.comments || []).map(walk);
    return s.concat.apply(s, _toConsumableArray(r));
  }

  return walk(thread);
}

var ChangeEmitter = (function (_EventEmitter) {
  _inherits(ChangeEmitter, _EventEmitter);

  function ChangeEmitter(opts) {
    _classCallCheck(this, ChangeEmitter);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(ChangeEmitter).call(this));

    _this.db = new _db2.default(opts.dbFilename);
    _this.client = new _FBClient2.default(opts.accessToken);
    _this.feedPath = opts.groupID + '/feed';
    _this.group = {
      id: opts.groupID,
      type: 'group',
      mods: opts.mods
    };

    _this.stop = (0, _refresher2.default)(opts.interval, function () {
      return _this.client.get(_this.feedPath, REFRESH_PARAMS).then(function (data) {

        var threads = normalizeResponse(data, _this.group);
        return Promise.all(_lodash2.default.flatten(threads.map(function (thread) {
          return walkThread(thread, function (post) {
            return _this.db.touchVersion(post).then(function (change) {
              if (change) {
                var state = _this.db.threadStateObj(post);
                return [change, post, state];
              } else {
                return null;
              }
            });
          });
        }))).then(function (p) {
          var _iteratorNormalCompletion = true;
          var _didIteratorError = false;
          var _iteratorError = undefined;

          try {
            for (var _iterator = _lodash2.default.compact(p)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              var e = _step.value;

              _this.emit.apply(_this, _toConsumableArray(e));
            }
          } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
              }
            } finally {
              if (_didIteratorError) {
                throw _iteratorError;
              }
            }
          }
        });
      });
    }, log.error);

    return _this;
  }

  return ChangeEmitter;
})(_events.EventEmitter);

exports.default = ChangeEmitter;