'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        * A thin wrapper around Facebook's Graph API that uses ES6 Promises.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        * @module
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        */

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * The base URL for all Graph API requests.
 * @constant {string}
 */
var BASE_URL = 'https://graph.facebook.com/v2.3/';

/**
 * Whether or not dangerous Graph API requests will actually be
 * executed. If true, POST, PUT, and DELETE will be noops.
 * @constant {boolean}
 */
var SIMULATE = false;

/**
 * Returns the Graph API URL for a query.
 * @param {string} path
 * @param {Object=} params
 * @return {string}
 */
function urlFor(path, params) {
  var queryString = _qs2.default.stringify(params || {});
  return BASE_URL + path + '?' + queryString;
}

/**
 * Makes a request to the Graph API.
 * @param {string} accessToken
 * @param {string} method
 * @param {string} path
 * @param {Object=} params
 * @return {Promise}
 */
function graphRequest(accessToken, method, path) {
  var params = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

  params = Object.assign({}, params, { access_token: accessToken });

  return new Promise(function (resolve, reject) {
    (0, _request2.default)({
      method: method,
      uri: urlFor(path, params),
      json: true
    }, function (err, res, body) {
      if (err) {
        return reject(err);
      }
      if (body.error) {
        return reject(new Error(body.error.message));
      }
      resolve(body);
    });
  });
}

var FBClient = (function () {

  /**
   * A Facebook Graph API client.
   * @class
   * @param {string} accessToken
   */

  function FBClient(accessToken) {
    _classCallCheck(this, FBClient);

    this.accessToken = accessToken;
  }

  /**
   * Makes a GET request to the Graph API.
   * @param {string} path
   * @param {Object=} params
   * @return {Promise}
   */

  _createClass(FBClient, [{
    key: 'get',
    value: function get(id, params) {
      return graphRequest(this.accessToken, 'GET', id, params);
    }

    /**
     * Makes a PUT request to the Graph API.
     * @param {string} path
     * @param {Object=} params
     * @return {Promise}
     */

  }, {
    key: 'put',
    value: function put(id, params) {
      if (SIMULATE) {
        return this.noop();
      }
      return graphRequest(this.accessToken, 'PUT', id, params);
    }

    /**
     * Makes a DELETE request to the Graph API.
     * @param {string} path
     * @param {Object=} params
     * @return {Promise}
     */

  }, {
    key: 'del',
    value: function del(id, params) {
      if (SIMULATE) {
        return this.noop();
      }
      return graphRequest(this.accessToken, 'DELETE', id, params).catch(function (x) {
        return x;
      });
    }

    /**
     * Makes a POST request to the Graph API.
     * @param {string} path
     * @param {Object=} params
     * @return {Promise}
     */

  }, {
    key: 'post',
    value: function post(id, params) {
      if (SIMULATE) {
        return this.noop();
      }
      return graphRequest(this.accessToken, 'POST', id, params);
    }

    /**
     * Returns a promise that does nothing and resolves to an empty
     * object. Useful for testing.
     * @return {Promise}
     */

  }, {
    key: 'noop',
    value: function noop() {
      return Promise.resolve({});
    }
  }]);

  return FBClient;
})();

exports.default = FBClient;