#! /usr/bin/env node
'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _log = require('./log');

var _log2 = _interopRequireDefault(_log);

var _ChangeEmitter = require('./ChangeEmitter');

var _ChangeEmitter2 = _interopRequireDefault(_ChangeEmitter);

var _record = require('./record');

var _record2 = _interopRequireDefault(_record);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

process.on('uncaughtException', function (ex) {
  console.error(ex);
  console.log('Exiting due to uncaught exception...');
  process.exit(1);
});

process.on('unhandledRejection', function (reason, p) {
  p.catch(function (err) {
    console.error(err.stack);
    console.log('Exiting due to unhandled rejection...');
    process.exit(1);
  });
});

process.on('SIGINT', function () {
  console.log('Exiting...');
  process.exit(0);
});

var CLIArgs = (function () {
  function CLIArgs(args) {
    _classCallCheck(this, CLIArgs);

    this.obj = {};
    var nextPos = 0;
    var flag = null;
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = args[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var a = _step.value;

        if (a.indexOf('-') === 0) {
          this.obj[a] = this.obj[a] || [];
          flag = a;
        } else if (flag == null) {
          this.obj[nextPos] = (this.obj[nextPos] || []).concat([a]);
          nextPos++;
        } else {
          this.obj[flag] = this.obj[flag].concat([a]);
          flag = null;
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  }

  _createClass(CLIArgs, [{
    key: 'asRaw',
    value: function asRaw() {
      var _this = this,
          _ref;

      for (var _len = arguments.length, flags = Array(_len), _key = 0; _key < _len; _key++) {
        flags[_key] = arguments[_key];
      }

      var allArgs = flags.map(function (f) {
        return _this.obj[f.toString()] || [];
      });
      return (_ref = []).concat.apply(_ref, _toConsumableArray(allArgs));
    }
  }, {
    key: 'asString',
    value: function asString() {
      return this.asRaw.apply(this, arguments).join('');
    }
  }, {
    key: 'asInteger',
    value: function asInteger() {
      var ints = this.asRaw.apply(this, arguments);
      if (ints.length === 0) {
        return Number.NaN;
      } else {
        return parseInt(ints[0], 10);
      }
    }
  }, {
    key: 'asStringArray',
    value: function asStringArray() {
      var res = this.asRaw.apply(this, arguments).join(',').split(',');
      return res[0] === '' ? [] : res;
    }
  }]);

  return CLIArgs;
})();

var CLI = (function () {
  function CLI(args) {
    _classCallCheck(this, CLI);

    if (args.length === 0 || args.indexOf('-h') !== -1 || args.indexOf('--help') !== -1) {

      this.mode = this.help;
    } else {

      this.mode = this.watch;
      this.opts = {
        interval: 5000,
        accessToken: null,
        groupID: null,
        mods: [],
        scripts: [],
        dbFilename: 'hackbot.db',
        recordDir: 'record'
      };

      this.processArgs(args);
    }
  }

  _createClass(CLI, [{
    key: 'processArgs',
    value: function processArgs(args) {

      var cliArgs = new CLIArgs(args);

      this.opts.groupID = cliArgs.asString(0);
      if (this.opts.groupID === '') {
        _log2.default.error('you must provide a Facebook Group ID');
        process.exit(1);
      }

      this.opts.accessToken = cliArgs.asString(1);
      if (this.opts.accessToken === '') {
        _log2.default.error('you must provide a Facebook access token');
        process.exit(1);
      }

      var i = cliArgs.asInteger('-i', '--interval');
      if (!isNaN(i) && i > 0) {
        this.opts.interval = i;
      }

      this.opts.mods = cliArgs.asStringArray('-m', '--mod', '--mods');
      if (this.opts.mods.length === 0) {
        _log2.default.error('you must provide at least one moderator ID using -m');
        process.exit(1);
      }

      this.opts.scripts = cliArgs.asStringArray('-s', '--script', '--scripts');
      if (this.opts.scripts.length === 0) {
        _log2.default.error('you must provide at least one script using -s');
        process.exit(1);
      }

      var d = cliArgs.asString('-d', '--database');
      if (d !== '') {
        this.opts.dbFilename = d;
      }

      var r = cliArgs.asString('-r', '--record');
      if (d !== '') {
        this.opts.recordDir = r;
      }
    }
  }, {
    key: 'run',
    value: function run() {
      this.mode.apply(this);
    }
  }, {
    key: 'help',
    value: function help() {
      (0, _log2.default)(['usage: hackbot <group> <access-token> [<args>...]'].join('\n'));
    }
  }, {
    key: 'watch',
    value: function watch() {
      var emitter = new _ChangeEmitter2.default(this.opts);
      (0, _log2.default)('started with database', { filename: this.opts.dbFilename });

      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = this.opts.scripts[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var script = _step2.value;

          try {
            (0, _log2.default)('loading script', { name: script });

            var _require = require('./scripts/' + script);

            var attach = _require.attach;

            attach(emitter, (0, _record2.default)(this.opts.recordDir));
          } catch (ex) {
            _log2.default.error('no such script', { name: script });
            process.exit(1);
          }
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    }
  }]);

  return CLI;
})();

exports.default = CLI;

var cli = new CLI(process.argv.slice(2));
cli.run();