'use strict';

var _child_process = require('child_process');

var _child_process2 = _interopRequireDefault(_child_process);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _log = require('./log');

var _log2 = _interopRequireDefault(_log);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function mkdirP(path) {
  return new Promise(function (resolve, reject) {
    _child_process2.default.exec('mkdir -p ' + path, function (err, stdout, stderr) {
      if (err) {
        reject(err);
      } else {
        resolve();
      }
    });
  });
}

function echo(path, str) {
  return new Promise(function (resolve, reject) {
    _fs2.default.writeFile(path, str, function (err) {
      if (err) {
        reject(err);
      } else {
        resolve();
      }
    });
  });
}

function addCommitPush(recordDir, filename) {
  return new Promise(function (resolve, reject) {
    _child_process2.default.exec('cd ' + recordDir + ' && git add ' + filename + ' && git commit -m \'' + filename + '\' && git push origin master', function (err, stdout, stderr) {
      if (err) {
        reject(err);
      } else {
        resolve();
      }
    });
  });
}

function dirExists(filename) {
  return new Promise(function (resolve, reject) {
    _fs2.default.stat(filename, function (err, stat) {
      if (err == null) {
        resolve(true);
      } else if (err.code === 'ENOENT') {
        resolve(false);
      } else {
        reject(err);
      }
    });
  });
}

module.exports = function (recordDir) {
  return function (type, data) {
    var occurred = arguments.length <= 2 || arguments[2] === undefined ? (0, _moment2.default)() : arguments[2];

    var dateStr = occurred.format('YYYY-MM-DD-hh-mm-ss');
    var filename = dateStr + '-' + type + '.json';
    var fullPath = _path2.default.resolve(recordDir, filename);
    (0, _log2.default)('recording \'' + type + '\' action to ' + fullPath);

    return mkdirP(recordDir).then(function () {
      return echo(fullPath, JSON.stringify(data, function (k, v) {
        if (k === 'parent' || k === 'group') {
          return v.id;
        } else if (k === 'fromName' && !data.isMod) {
          return 'Anonymous';
        } else {
          return v;
        }
      }, 2));
    }).then(function () {
      return dirExists(_path2.default.resolve(recordDir, '.git'));
    }).then(function (isGit) {
      if (isGit) {
        return addCommitPush(recordDir, filename);
      }
    });
  };
};