'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.attach = attach;
/** @module */

var CLOSE_COMMANDS = ['close', 'lock', 'thread'];

function isClosePost(post) {
  return post.command != null && CLOSE_COMMANDS.indexOf(post.command) !== -1;
}

/**
 * Allows moderators to close comment threads. Looks for the first comment by a
 * moderator to issue the "/thread" command, and automatically deletes all
 * non-moderator posts after it.
 */
function attach(emitter, record) {

  emitter.on('new', function (post, state) {

    if (!post.isMod) {
      state.get().then(function (s) {
        if (s.closed) {
          emitter.client.del(post.id).then(function () {
            record('close-delete', { post: post });
          });
        }
      });
    } else if (isClosePost(post)) {
      record('close', { post: post });
      state.update({ closed: true });
    }
  });
}