'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.attach = attach;
/** @module */

var DELETE_COMMANDS = ['delete', 'mod', 'moderate'];

function isDeletePost(post) {
  return post.command != null && DELETE_COMMANDS.indexOf(post.command) !== -1;
}

/**
 * Allows moderators to close comment threads. Looks for the first comment by a
 * moderator to issue the "/delete" command, and automatically deletes all
 * non-moderator posts after it.
 */
function attach(emitter, record) {

  emitter.on('new', function (post, state) {
    if (post.isMod && isDeletePost(post) && post.level > 0) {
      (function () {
        var parent = post.parent;
        emitter.client.del(parent.id).then(function () {
          record('delete', { post: parent });
        });
      })();
    }
  });
}